<?php
namespace Teknisa\OPL\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DTO\Request\Row;
use \Zeedhi\Framework\DTO\Response\Message;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource;
use Exception;
use \Teknisa\OPL\Services\Dfs48000SaidaDoOperadorLogisticoService;
    
class Dfs48000SaidaDoOperadorLogisticoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    
    public function __construct($crudService, $parameterBag, Dfs48000SaidaDoOperadorLogisticoService $service) {
        $this->crudService  = $crudService;
        $this->service      = $service;
        $this->parameterBag = $parameterBag;
    }
    
    public function getDtProcessaOpl(Row $request, Response $response) {
        try {
            $params = $request->getRow();
      
            $result = $this->service->getDtProcessaOpl($params);
            $dtprocessa = $this->service->getDtProcessaAtual($params);
            
            $result[] = isset($dtprocessa) ? $dtprocessa[0] : null;
            
            $response->addDataSet(new DataSet('res', $result));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function validaItensPedido(Row $request, Response $response) {
        try {
            $rows = $request->getRow();
            $dados = $rows['DADOSGERACAO'];
            $pedidos = $rows['NFS'];
            $retornos = array();
            $pedComErro = array();
            $pedSemErro = array();
            $pedidosParaSaida = array();
            $retornoSaida;
            
            $dtprocessa = $this->service->getDtProcessaAtual($dados);
            $stwebservice = $this->service->getStWebService($dados);
            
            if(!empty($stwebservice)) {
                $dados['WEBSERVICE'] = 'S';
            } else {
                $dados['WEBSERVICE'] = 'N';
            }
            
            $dados['DTPROC'] = $dtprocessa[0]['DTPROCESSA'];
            
            foreach($pedidos as $ped) {
                $result = $this->service->validaItensPedido($ped, $dados);
                
                if(!empty($result)) {
                    foreach($result as $res) {
                        if($res['LISTAERRO'] != NULL) {
                            $strErro = '';
                            $codErros = explode('_', $res['LISTAERRO']);
                            $codErros = array_filter($codErros, static function($e) {
                                return $e != "";
                            });
                            
                            foreach($codErros as $err) {
                                if($strErro != '')
                                    $strErro .= "\n";
                                    
                                switch($err) {
                                    case 'PRODINATIVO':
                                        $strErro .= "Produto Inativo para o OL.";
                                        break;
                                    case 'ERROVLR':
                                        $strErro .= "Produto com Custo menor ou igual a zero.";
                                        break;
                                    case 'ERRORES':
                                        $strErro .= "Produto sem Reserva.";
                                        break;
                                    case 'ERROQTD':
                                        $strErro .= "Quantidade atendida pelo OL deve ser informada.";
                                        break;
                                    case 'ERRORELAC':
                                        $strErro .= "Produto sem relacionamento com produto do OL.";
                                        break;
                                    case 'ERROEST':
                                        $strErro .= "Sem quantidade disponível no estoque do OL.";
                                        break;
                                    case 'ERROCX':
                                        $strErro .= "Produto caixa inativo.";
                                        break;
                                }
                            }
                            
                            $res['NMERRO'] = $strErro;
                            
                            $retornos[] = $res;
                            
                            if(!in_array($res['NRAUTOFORN'], $pedComErro)) {
                                $pedComErro[] = $res['NRAUTOFORN'];
                            }
                        }
                    }
                    
                    if(count($pedComErro) > 0) {
                        foreach($pedidos as $ped) {
                            if(!in_array($ped['NRAUTOFORN'], $pedComErro)) {
                                if(!in_array($ped['NRAUTOFORN'], $pedSemErro)) {
                                    $pedSemErro[] = $ped['NRAUTOFORN'];
                                }
                            }
                        }
                    }
                }
            }
            
            if(count($pedSemErro)) {
                foreach($pedSemErro as $pse) {
                    foreach($pedidos as $ped) {
                        if($ped['NRAUTOFORN'] == $pse) {
                            $pedidosParaSaida[] = $ped;
                        }
                    }
                }
            }
            
            if(count($pedidosParaSaida) > 0) {
                // chama toda parte de geracao de notas para os pedidos sem erros
                $retornoSaida = $this->gerarSaidaPedidos($pedidosParaSaida, $dados);
            } else {
                $retornoSaida = array(
                    'ERROS'         => array(),
                    'NOTAS_GERADAS' => array()
                );
            }
            
            $response->addDataSet(new DataSet('saida', $retornoSaida));
            $response->addDataSet(new DataSet('pedErros', $retornos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function gerarSaidaPedidos($pedidos, $dados) {
        $retorno = array(
            'ERROS'         => array(),
            'NOTAS_GERADAS' => array()
        );
        
        /*$nrLanctoNfNota = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_NOTA_LANC.$cdFilDest, $organizacao); //  LANCNF/FILIAL
        //var_dump($nrLanctoNfNota);die;
        
        $nrLancEstoq    = $this->NewCode->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$cdFilDest, $organizacao); //LANCTOEST
        //var_dump($nrLancEstoq);die;
        
        // Gera o corpo da nota fiscal de devolução
        $proxLancamento = $this->lancamentoNotaFiscalService->geraCorpoNota($cdFilDest, $cdFilDest, $paramsGeracao["CDEMPRESA"], $dadosDevolucao["CDTIPOOPER"], $cdSerie, $cdFilEmit, 'DFE',
        'S', $nrLanctoNfNota, '', '', '', '', '', '', '', '', '', $nrLancEstoq, $dadosDevolucao["DTSAIDA"], 0, 0, '', null);
        //var_dump($proxLancamento);die;
        $this->dfs48000SaidaDoOperadorLogisticoFactory->updateNotaDevol($cdFilDest, 'S', $proxLancamento["P_PRXLANCTONF"]);
        //var_dump($proxLancamento["P_PRXLANCTONF"]);die;
        $this->dfs48000SaidaDoOperadorLogisticoFactory->insereItemNota($cdFilDest, $proxLancamento["P_PRXLANCTONF"], '000', $notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $organizacao);
        
        //var_dump($notaFiscal["CDFILIAL"]);die;
        foreach ($produtos as $produto) {
            try {
                //var_dump($produtos);die;
                $dtPosicao = date('d/m/Y', strtotime("+1 days", strtotime(str_replace("/", "-", $dadosDevolucao["DTSAIDA"]))));
                $posicaoEstoqueProd = $this->dfs48000SaidaDoOperadorLogisticoFactory->getQtEstoqDia($cdFilDest, $produto["CDPRODUTO"], $dtPosicao);
                //var_dump($dtPosicao);die;
                //$this->validaProduto($produto, $notaFiscal, $posicaoEstoqueProd[0]);
                
                $cdProdEmis = $this->maskProduto($produto["CDARVPROD"], $organizacao);
                //var_dump($produto["CDARVPROD"]);die;
                
                $this->lancamentoNotaFiscalService->procedureGeraNotaFiscalProduto($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $produto["CDPRODUTO"], $cdProdEmis, $produto["NMPRODUTO"], $produto["QTDEVOLUCAOOL"], $produto["VRUNITITEM"], 0, '', '', '', '', '', 'N', 'S', 0, $produto["CDMARCA"]);
            
                // Atualiza os produtos que deram origem a devolução
                $this->dfs48000SaidaDoOperadorLogisticoFactory->alteraProdutoOrigDev($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $produto["NRSEQITPROD"]);
                
                // Atualiza os produtos da nota de devolução 
                $this->dfs48000SaidaDoOperadorLogisticoFactory->alteraProdutoDevolucao($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMOTIVODEV"]);
                
                // Gera ocorrência de atendimento(GQL)
                //if($produto["CDMOTIVODEV"]) $this->geraOcorrenciaDeAtendimento($produto, $notaFiscal, $dadosDevolucao, $proxLancamento, $cdOperador, $nmOperador, $cdFilDest, $cdFilEmit, $organizacao);
                
                if(isset($paramSupri[0])) {
                    if($paramSupri[0]["IDUTILCONSSOLIOL"] == 'N') {
                        $this->atualizaStatusAF($produto, $notaFiscal, $dadosDevolucao, $cdFilDest);
                    }
                }
                
            }
            catch(\Exception $e) {
                $geraNota = false;
                array_push($this->arrayProdutosErro, array("CDFILIAL" => $cdFilDest,
                  "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
                  "DTENTRSAID" => $notaFiscal["DTENTRSAID"],
                  "DTENTRADA"  => $dadosDevolucao["DTSAIDA"],
                  "NMPRODUTO"  => $produto["NMPRODUTO"],
                  "DSERRO"     => $e->getMessage()));
            }
        
        }
        
        if($geraNota) {
            // Insere os dados adicionais para a nota
            $this->insereDadosAdicionais($notaFiscal, $proxLancamento, $cdFilDest, $cdFilEmit, $dadosDevolucao, $dadosAdicionaisNota, $organizacao);
           
            // Realiza o Fechamento da nota
            $this->lancamentoNotaFiscalService->procedureGeraFechamentoSaida($cdFilDest, $proxLancamento["P_PRXLANCTONF"], substr($paramsGeracao["DTPROCESSA"], 0, 10), $proxLancamento["P_PRXLANCESTQ"]);
            
            // Verifica se a operação atualiza estoque
            // if($this->operacaoAtualizaEstoque($dadosDevolucao["CDTIPOOPER"], 'S')) {
            //     if($paramsGeracao["IDESTONLINE"] == 'S') {
            //         // A função atualiza estoque também busca por irregularidades
            //         $this->validacoesEstoqueService->atualizaEstoque($cdFilDest, $proxLancamento["P_PRXLANCESTQ"], true, true, null, $dadosDevolucao["DTSAIDA"]);
            //     }
            // }
            
            $cdModDocFisc = $this->dfs48000SaidaDoOperadorLogisticoFactory->getCdModDocFisc($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
            if($cdModDocFisc == null) throw new \Exception(self::MSG_ERROS["SEMDOCUFISC"]);
            
            $produtos     = $this->dfs48000SaidaDoOperadorLogisticoFactory->getItemProdutoNota($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
            
            // foreach($produtos as $produto) {
            //     try {
            //         if(!isset($produto["CDFISOPEITEM"])) {
            //             $geraNota = false;
            //             throw new \Exception(self::MSG_ERROS["ITEMSEMCFOP"]);
            //         }
            //     }
            //     catch(\Exception $e) {
            //         $geraNota = false;
            //         array_push($this->arrayProdutosErro, array("CDFILIAL" => $cdFilDest,
            //           "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
            //           "DTENTRSAID" => $notaFiscal["DTENTRSAID"],
            //           "DTENTRADA"  => $dadosDevolucao["DTSAIDA"],
            //           "NMPRODUTO"  => $produto["NMPRODUTO"],
            //           "DSERRO"     => $e->getMessage()));
            //     }
            // }
            
            if($geraNota) {
                $this->verififcaImpostosObrigatorio($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);

                $this->geraRateio($cdFilDest, $proxLancamento["P_PRXLANCTONF"], 'S', '', $organizacao);
                
                $this->notasSucesso += 1;
                $this->connection->commit();
                
            }
            else $this->connection->rollback();
        }*/
        
        return $retorno;
    }
}
